#include "solver.h"
#include "evaluator.h"
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <time.h>
#include "boost/numeric/ublas/io.hpp"

using namespace fmi;
using namespace std;

int main( int argc, char *argv[] )
{

  
   int exitstatus = 0;

   // Reserve memory for the number of points

   int numberOfPoints = 4000;

   if(argc >= 2)
      numberOfPoints = atoi(argv[1]);

//   std::valarray<double> values(numberOfPoints);
//   std::valarray<double> points(3*numberOfPoints);
   std::vector<double> values(numberOfPoints);
   std::vector<double> points(3*numberOfPoints);


//   srand48((unsigned int)time(0));

	

   for(int n=0;n<numberOfPoints;n++)
   {

      double p[3];
      double f = 100.0;
	     
      p[0] = drand48()*f;
      p[1] = drand48()*f;
      p[2] = drand48()*f;
      points[3*n] = p[0];
      points[3*n+1] = p[1];
      points[3*n+2] = p[2];
	     
      values[n] = drand48()*f;

	     
      std::cout << p[0] << " " << p[1] << " " << p[2] 
		<< " " << values[n] << "\n";
	     
   }
	

	
   solver solv;
   solv.SetPoints(&points[0],numberOfPoints);
   solv.SetValues(&values[0],numberOfPoints);
   solv.PreconditioningOn();
//   solv.PreconditioningOff();
   solv.SetMaxIterations(40);
   solv.SetRestartIterations(40);
   solv.SetPolynomialDegree(-1);
   solv.SetK(1);
   //solv.SetTau( std::pow(1.0/numberOfPoints,1.0/3.0) );
   solv.SetTau( 0.0 );
   solv.SetTolerance(1e-8);

   solv.Update();

   //std::cout << solv.GetSolution() << std::endl;

   typedef std::vector< ScalarType > VectorType;
   VectorType result(numberOfPoints);

   double elapsedTimeFMM,elapsedTimeDirect;
   
//for(int dep=3;dep<=3;dep++)
//for(int pp=3;pp<=15;pp++)
   {
      //int dep = 2;
      int pp = 15;
      cout << endl << "P: " << pp << endl;
      //cout << "depth: " << dep << endl;
	
      unsigned int start_time, end_time;
      double elapsed_time;
	

      evaluator eval;

      eval.SetPointSet(&points[0],numberOfPoints);
      //eval.SetExtraPolynomial( solv.GetPolynomialDegree() );
      eval.SetCoefficients( solv.GetSolution(),numberOfPoints );
      eval.SetExtraPolynomial( -1,0 );
      //double extent[6] = {-.1, 1.1, -.1, 1.1, -.1, 1.1};
      //eval.SetBoundingBox(extent);
      eval.SetP(pp);
      eval.SetK(1);
      eval.SetDepthToOptimal();
      //eval.SetTau( std::pow(1.0/numberOfPoints,1.0/3.0) );
      //eval.SetTau( solv.GetTau() );
      eval.SetTau( 0.0 );
      eval.SetMinDegree(6);
      eval.SetWellSeparatedFactor(2.0);
      eval.SetDecreaseFactor(1);
      eval.SetDecreaseFactor2(0);
   
      start_time = clock();

      eval.Update();

      eval.EvaluateAtAllSourcePoints(&result[0]);
      //eval.EvaluateAtAllSourcePointsNew(&result[0]);

      end_time = clock();

      elapsedTimeFMM = elapsed_time = 
	 (end_time - start_time )/(double)CLOCKS_PER_SEC;
      //std::cout << " Run time = " << (t2 - t1 - overhead) << " cycles = ";
      cout << elapsed_time       << " sec  = ";
      cout << (elapsed_time/60.0) << " min = ";
      cout << (elapsed_time/3600.0) << " hr" << endl;



      //ScalarType r,r2,average_error = 0.0;
      double r,r2,average_error = 0.0;
      //ScalarType average_error2 = 0.0;
      double max_error = 0.0;
      double min_error = 1e30;
      //ScalarType max_error2 = 0.0;

	
      average_error = 0.0;
      //average_error2 = 0.0;
      max_error = 0.0;
      //max_error2 = 0.0;
      //if(1)
      for(int i=0;i<numberOfPoints;i++)
      {


	 //r2 = result[i];
	 r = eval.EvaluateDirect(&points[3*i]);
	 r2 = values[i];

	 //std::cout << r2 << " ";
	 //if(fabs(r-r2) > 1)
	 std::cout << r << "\n";
	 //std::cout << "error \n";
	
	 average_error += fabs(r-r2);
	 //average_error2 += fabs(r-r3);

	 if( fabs(r-r2) > max_error )
	    max_error = fabs(r-r2);
	 if( fabs(r-r2) < min_error )
	    min_error = fabs(r-r2);
	 //if( fabs(r-r3) > max_error2 )
	 // max_error2 = fabs(r-r3);
      
      }

      average_error /= numberOfPoints;
      //average_error2 /= numberOfPoints;
	
      cout << endl << "average_error: " << average_error << endl;
      cout << "max_error: " << max_error << endl;
      cout << "min_error: " << min_error << endl;




   }

   return exitstatus;
   
}
